function [mu] = exactregl1(probname,A,b,c,K,fid,fidits)
% [MU] = EXACTREGL1(PROBNAME,A,B,C,K,FID,FIDITS)
%
% Take a convex optimization problem in SeDuMi format and solve its
% exact l1 regularization.
%
% INPUTS:
%
% PROBNAME  Name of the problem (string).  Used only for the log.
% A         is an m-by-n matrix.
% B         is an m-vector (the RHS).
% C         is an n-vector (the objective).
% K         is a structure that describes the conic constraints.
% FID       is the file ID used to write the summary output (optional).
% FIDITS    is the sifle ID used to write SeDuMi's output (optional).
%
% OUTPUTS:
%
% mu        is the Lagrange multiplier of the selection problem.
%
% Authors:
% Michael P. Friedlander        Paul Tseng
% Univ of British Columbia      Univ of Washington
% mpf@cs.ubc.ca                 tseng@math.washington.edu
% 
% $Revision: 106 $ $Date: 2006-11-22 17:52:19 -0800 (Wed, 22 Nov 2006) $

  n       = length(c);
  m       = length(b);
  nzeros  = zeros(n,1);
  nneye   = speye(n);
  if nargin < 6 || isempty(fid   ), fid    = 1; end
  if nargin < 7 || isempty(fidits), fidits = 1; end

% Parameters for Sedumi.
  params.fid = fidits;
  
% Print log header.
  fprintf(fidits,' ================================================\n');
  fprintf(fidits,' %-10s n = %5i  m = %5i  nnzA = %5i\n',probname,n,m,nnz(A));
  fprintf(fidits,' ================================================\n');
  fprintf(fid   ,' ================================================\n');
  fprintf(fid   ,' %-10s n = %5i  m = %5i  nnzA = %5i\n',probname,n,m,nnz(A));
  fprintf(fid   ,' ================================================\n');
  fprintf(fid,' %5s  %4s  %9s  %9s  %8s  %7s  %7s  %7s  %8s  %3s\n',...
              'Prob','its','c''x','gap','Norm1',...
              'Sparse1','Sparse2','Sparse3','mu','Err');
  logH = ' %5s  %4i  %9.2e  %9.2e  %8.2e  %7i  %7i  %7i  %8.2e  %1i%1i%1i\n';
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The original (unregularized) problem.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  [x1, y1, info1] = sedumi(A, b, c, K, params);

  obj1     = c'*x1;
  r1       = relgap(b,c,x1,y1);

  fprintf(fid,logH,'Orig',info1.iter,...
          obj1,             ...
          r1,               ...
          norm (x1, 1    ), ...                 % 1-norm of x
          norm0(x1, r1   ), ...                 % 0-norm of x
          norm0(x1, r1*10), ...                 % ...
          norm0(x1, r1/10), ...                 % ...
          0,                ...                 % No mu
          info1.pinf,info1.dinf,info1.numerr);  % misc
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The selection problem.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Make sure these numbers exist.  
  if ~isfield(K,'f') || isempty(K.f), K.f = 0; end
  if ~isfield(K,'l') || isempty(K.l), K.l = 0; end

  nF     = K.f;                      % No. of free vars.
  nR     = n - nF;                   % No. of constrained vars.
  
  free   = 1 : nF;                   % Indices of free vars in x.
  cons   = nF+1 : n;                 % Indices of constrained vars in x.  

% Location of variables in augmented vector.
                     t=0    ;
  iF   = t+1:t+ nF ; t=t+ nF;      % x(free): nF
  is   = t+1:t+  1 ; t=t+ 1 ;      % is     : 1
  iv   = t+1:t+  n ; t=t+ n ;      % iv     : n
  iw   = t+1:t+  n ; t=t+ n ;      % iw     : n
  iR   = t+1:t+ nR ; t=t+ nR;      % x(cons): nR 
  ix   = [iF iR];                  % x      : n
  
  K2   = K;
  K2.l = K.l + 1 + 2*n;  % Make space for s(1) and v(n),w(n).

% Assemble the constraints.  Note that Sedumi requires the conic
% constraints to list free vars followed by nonnegative vars.  The
% variables and constraints are then ordered as
%
% A(:,free)x(free)          + A(:,cons)x(cons) = b  : A x            = b
%          x(free) - v +  w +          x(cons) = 0  :   x    - v + w = 0
% c(  free)x(free)          + c(  cons)x(cons) = p* : c'x + s        = p*
%                                                           s >= 0.

%          x(free)     s(1)        v(n)         w(n)         x(cons)
  c2 = [ nzeros(free) ;   0       ; ones(n,1)  ; ones(n,1)  ; nzeros(cons) ];
  A2 = [ A(:,free)    ,sparse(m,1), sparse(m,n), sparse(m,n), A(:,cons)
         nneye(:,free),sparse(n,1),-speye(n)   , speye(n)   , nneye(:,cons)
         c(free)'     ,   1       , zeros(1,n) , zeros(1,n) , c(cons)'     ];

  b2 = [ b          % (m)
         nzeros     % (n)
         obj1   ];  % (1)
  
% Solve.
  [xvw2, y2, info2] = sedumi(A2, b2, c2, K2, params);
  
% Pull out the solution xphi and the Lagrange multiplier mu.
  mu       = - y2(end);     % The slack var s guarantees ybar(end) <= 0.
  x2       = xvw2(ix);
  v2       = xvw2(iv);
  w2       = xvw2(iw);
  r2       = relgap(b2,c2,xvw2,y2);
  obj2     = c'*x2;         % Verify that we recovered the original obj.

% Print log.
  fprintf(fid,logH,'Selec',info2.iter,...
          obj2,                ...              % objective
          r2,                  ...              % relative gap
          norm (x2, 1    ),    ...              % 1-norm of x
          norm0(x2, r2   ),    ...              % 0-norm of x
          norm0(x2, r2*10),    ...              % ...
          norm0(x2, r2/10),    ...              % ...
          mu,                  ...              % Lagrange mult
          info2.pinf,info2.dinf,info2.numerr);  % misc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The regularized problem.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  delta = 0.5 / mu;

% The rest is very similar to the selection problem, except that we need to
% eleiminate the slack variable.  We could force s=0 (ie, a fixed
% variable), but that might cause other difficulties.  Getting rid of s is
% clunky but direct.

% Location of variables in augmented vector.
                     t=0    ;
  iF   = t+1:t+ nF ; t=t+ nF;      % x(free): nF
  iv   = t+1:t+  n ; t=t+ n ;      % iv     : n
  iw   = t+1:t+  n ; t=t+ n ;      % iw     : n
  iR   = t+1:t+ nR ; t=t+ nR;      % x(cons): nR 
  ix   = [iF iR];                  % x      : n
  
  K3   = K;
  K3.l = K.l + 2*n;  % Make space for v(n),w(n).

% Assemble the constraints.  Note that Sedumi requires the conic
% constraints to list free vars followed by nonnegative vars.  The
% variables and constraints are then ordered as
%
% A(:,free)x(free)          + A(:,cons)x(cons) = b  : A x         = b
%          x(free) - v +  w +          x(cons) = 0  :   x - v + w = 0

%        x(free)        v(n)         w(n)         x(cons)
  c3 = [ c(free)      ; ones(n,1)  ; ones(n,1)  ; c(cons)];
  A3 = [ A(:,free)    , sparse(m,n), sparse(m,n), A(:,cons)
         nneye(:,free),-speye(n)   , speye(n)   , nneye(:,cons) ];

  c3([iv iw]) = delta*c3([iv iw]);
  
  b3 = [ b         % (m)
         nzeros ]; % (n)

  [xvw3, y3, info3] = sedumi(A3, b3, c3, K3, params);

% Pull out the regualarized solution and elastics.
  x3     = xvw3(ix);
  r3     = relgap(b3,c3,xvw3,y3);
  obj3   = c'*x3; 
  
% Print log.
  fprintf(fid,logH,'Reg',info3.iter,...
          obj3,                ...              % objective
          r3,                  ...              % relative gap
          norm (x3, 1    ),    ...              % 1-norm of x
          norm0(x3, r3   ),    ...              % 0-norm of x
          norm0(x3, r3*10),    ...              % ...
          norm0(x3, r3/10),    ...              % ...
          0,                   ...              % No mu
          info3.pinf,info3.dinf,info3.numerr);  % misc
  
% Finish off the log.
  fprintf(fid,'\n');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Print the summary line and save solutions.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  fprintf(fid,...
          '%10s & %9s & %9s & %9s & %9s & %7s & %7s & %8s\n',...
          'Prob','Obj1','Obj3','Norm1','Norm3','0norm1','0norm3','delta');
  fprintf(fid,...
          '%10s & %9.2e & %9.2e & %9.2e & %9.2e & %7i & %7i & %8.1e\n',...
          probname, obj1, obj3, ...
          norm(x1,1), norm(x3,1), norm0(x1, r3*10), norm0(x3,r3*10),delta);

  b1 = b; c1 = c;
  save(probname,'mu',              ...
       'b1','c1','x1','y1',        ...
       'b2','c2','x2','y2','xvw2', ...
       'b3','c3','x3','y3','xvw3'  )
  
  end % function exactreg1

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PRIVATE FUNCTION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  function rgap = relgap(b,c,x,y)
  % Compute the relative duality gap.
  gap  = abs( c'*x - b'*y );
  rgap = gap / ( 1 + norm(c)*norm(x) + norm(b)*norm(y) );
  end

  function count = norm0(x, tol)
  % Compute the "0"-norm of a vector x, ie, the number of nonzero
  % elements in x.  An element x_i is considered as 0 if |x_i| < tol.
  count = sum( abs(x) > tol );
  end

