########################################################################
# $Revision: 107 $ $Date: 2006-11-22 17:52:47 -0800 (Wed, 22 Nov 2006) $
#
# Solve the linear program
#
#   minimize  c'x  subject to  Ax = b,  xL <= x <= xU,
#
# and its one-norm regularized version.  The maximum allowable
# regularization parameter (on the one-norm) is determined by solving
# a subsidiary LP, as described in
#
#   Michael P. Friedlander, "Exact regularization of linear programs",
#   Department of Computer Science Technical Report No. nnn-05,
#   Univ. of British Columbia.
#
# Author: Michael P. Friedlander
#         University of British Columbia
#         mpf@cs.ubc.ca
#
########################################################################

########################################################################
# Options.
########################################################################

option show_stats  1;        # Show some more detail.
option presolve    0;        # No presolve.
option solver                # Choose solver.
#      ipopt;
       cplex;
option ipopt_options  'outlev    = 1 '
;
option cplex_options  'crossover = 0 '
       		      'comptol   = 1e-10 '
;

########################################################################
# Parameters and variables.
########################################################################

# Problem dimensions.
param m    integer >  0;     # No. of rows.
param n    integer >  0;     # No. of cols.
param nnz  integer >  0;     # No. of nonzeros in A

# Fundamental sets.
set ROWS := {1..m};
set COLS := {1..n};
set NODES within {ROWS,COLS};# Constraint sparsity pattern.

# Problem data.
param problem_name symbolic; # Name of the problem.
param A  {NODES};            # Constraint matrix.
param b  {ROWS};             # RHS.
param c  {COLS};             # Objective vector.
param xL {COLS};             # Lower bound on x.
param xU {COLS};             # Upper bound on x.
param x0 {COLS};             # Starting point.

# Parameters.
param val_ctx;               # optimal value of original LP.
param val_ctx_one_norm;      # optimal value of regularized LP.
param val_ct_reg1;           # value of c'x for regularized soln.
param mustar    >= 0;        # (neg of) multiplier of c'x <= val_ctx.
param deltamax  >= 0;        # its inverse.
param delta     >= 0;        # regularization parameter.
param eps2 = sqrt(2.2204e-16);

# Variables.
var x   {j in COLS} >= xL[j], <= xU[j];
var xp  {COLS} >= 0;
var xm  {COLS} >= 0;

########################################################################
# Objectives.
########################################################################

#### Original objective:  c'x
minimize  obj_ctx:
   sum {j in COLS} c[j] * x[j];

#### Pure 1-norm: ||x||_1
minimize obj_one_norm:
   sum {j in COLS} (xp[j] + xm[j]);

#### One-norm regularized objective:
minimize obj_ctx_one_norm:
   sum {j in COLS}
  ( c[j] * x[j]  +  delta*(xp[j] + xm[j]) );

########################################################################
# Constraints.
########################################################################

#### Linear onstraints: Ax = b.
subject to Ax_eq_b {i in ROWS}:
   sum { (i,j) in NODES } A[i,j] * x[j] = b[i];

#### Splitting constraint: x = x(+) - x(-).
subject to x_eq_diff {j in COLS}:
   x[j] = xp[j] - xm[j];

#### Optimal value constraint:
subject to optimal_value:
   sum {j in COLS} c[j] * x[j] <= val_ctx;

########################################################################
# Problems.
########################################################################

#### Original LP definition
problem min_ctx:
   x,                # Variables:   xL <= x <= xU.
   obj_ctx,          # Objective:   c'x
   Ax_eq_b           # Constraint:  Ax = b
;

#### Minimum one-norm over optimal solution set.
problem min_one_norm:
   x, xp, xm,        # Variables:   xL <= x <= xU  and  xp,xm >= 0.
   obj_one_norm,     # Objective:   ||x||_1
   Ax_eq_b,          # Constraint:  Ax = b
   x_eq_diff,        # Constraint:  x = xp - xm.
   optimal_value     # Constraint:  c'x <= val_ctx.
;

#### One-norm reguarlized LP.
problem min_ctx_one_norm:
   x, xp, xm,        # Variables:   xL <= x <= xU  and  xp,xm >= 0.
   obj_ctx_one_norm, # Objective:   c'x + delta ||x||_1
   Ax_eq_b,          # Constraint:  Ax = b
   x_eq_diff         # Constraint:  x = xp - xm
;

########################################################################
# Solve the problems.
########################################################################

##################################################
# Read the LP data.
##################################################
data FNAME.dat;

##################################################
# --> Solve: ORIGINAL lp.
##################################################
printf "\n---------- Solving original LP.\n";

# Starting point.
let {j in COLS} x[j] := x0[j];

solve min_ctx;

if solve_result = "infeasible" then {
   printf "\n--- Infeasible problem! --\n";
   quit;
}

# Store the optimal LP value.
let val_ctx := obj_ctx;

# 0, 1, and 2-norm of original solution.
param norm0_x_lp;
param norm1_x_lp;
param norm2_x_lp;
let norm0_x_lp := card({j in COLS: abs(x[j]) > eps2});
let norm1_x_lp := sum  {j in COLS} abs(x[j]);
let norm2_x_lp := sum  {j in COLS} x[j]^2;

# Write out (unregularized) solution.
printf {j in COLS}: "%22.15e\n", x[j] > FNAME.solu;

##################################################
# --> Solve: Min perturbation over the optimal set.
##################################################
printf "\n---------- Minimizing the regularization term.\n";

# Starting point.
let {j in COLS} x [j] := x0[j];
let {j in COLS} xp[j] :=   max(0, x[j]);
let {j in COLS} xm[j] := - min(0, x[j]);

solve min_one_norm;

# Threshold value of regularization parameter.
let mustar   := - optimal_value.dual;
let deltamax := 1 / max( 1e-6, mustar );
let delta    := .5 * deltamax;

##################################################
# --> Solve: Regularized LP.
##################################################
printf "\n--------- Solving the regularized LP.\n";

# Starting point.
let {j in COLS} x [j] := x0[j];
let {j in COLS} xp[j] :=   max(0, x[j]);
let {j in COLS} xm[j] := - min(0, x[j]);

solve min_ctx_one_norm;

# Store the optimal value.
let val_ctx_one_norm := obj_ctx_one_norm;

# Store c' (regularized x).
let val_ct_reg1 := sum {j in COLS} c[j]*x[j];

# 0, 1, and 2-norm of regularized solution.
param norm0_x_reg1;
param norm1_x_reg1;
param norm2_x_reg1;
let norm0_x_reg1 := card({j in COLS: abs(x[j]) > eps2});
let norm1_x_reg1 := sum  {j in COLS} abs(x[j]);
let norm2_x_reg1 := sum  {j in COLS} x[j]^2;

# Write out (unregularized) solution.
printf {j in COLS}: "%22.15e\n", x[j] > FNAME.solr;

##################################################
# Generate some statistics

printf "\n--------- Solution summary.\n";
printf "\n";
printf "delta    = %12.5e\n", delta;
printf "deltamax = %12.5e\n", deltamax;
printf "\n";
printf "Problem               Objective      c'x       norm0       norm1       norm2\n";
printf "Original LP          %10.2e  %10.2e    %5i  %10.1e  %10.1e\n",
        val_ctx,
        val_ctx,
	norm0_x_lp,
	norm1_x_lp,
	norm2_x_lp;
printf "One-norm regularized %10.2e  %10.2e    %5i  %10.1e  %10.1e\n",
        val_ctx_one_norm,
	val_ct_reg1,
	norm0_x_reg1,
	norm1_x_reg1,
	norm2_x_reg1;
printf "\n\n";

#printf "%20s & %10s & %10s & %10s & %10s & %7s & %7s & %10s\n",
#       "Problem",
#       "val",
#       "c'xb",
#       "1_x",
#       "1_xb",
#       "0_x",
#       "0_xb",
#       "deltamax"
#       >> lp_sparse.out;
printf "\\\\%20s & %10.1e & %10.1e & %10.1e & %10.1e & %7i & %7i & %10.1e\n",
       problem_name,
       val_ctx,
       val_ct_reg1,
       norm1_x_lp,
       norm1_x_reg1,
       norm0_x_lp,
       norm0_x_reg1,
       deltamax
       >> lp_sparse.out;

